@@ -28,6 +28,7 @@ in this Software without prior written authorization from The Open Group.
 #include <config.h>
 #endif
 #include "Xlibint.h"
+#include "reallocarray.h"
 
 static void
 _XQueryColors(
@@ -37,14 +38,12 @@ _XQueryColors(
     int ncolors)
 {
     register int i;
-    xrgb *color;
     xQueryColorsReply rep;
-    long nbytes;
     register xQueryColorsReq *req;
 
     GetReq(QueryColors, req);
 
-    req->cmap = cmap;
+    req->cmap = (CARD32) cmap;
     SetReqLen(req, ncolors, ncolors); /* each pixel is a CARD32 */
 
     for (i = 0; i < ncolors; i++)
@@ -52,10 +51,11 @@ _XQueryColors(
        /* XXX this isn't very efficient */
 
     if (_XReply(dpy, (xReply *) &rep, 0, xFalse) != 0) {
-	if ((color = (xrgb *)
-	    Xmalloc((unsigned) (nbytes = (long) ncolors * SIZEOF(xrgb))))) {
+	xrgb *color = Xmallocarray((size_t) ncolors, sizeof(xrgb));
+	if (color != NULL) {
+            unsigned long nbytes = (size_t) ncolors * SIZEOF(xrgb);
 
-	    _XRead(dpy, (char *) color, nbytes);
+	    _XRead(dpy, (char *) color, (long) nbytes);
 
 	    for (i = 0; i < ncolors; i++) {
 		register XColor *def = &defs[i];
@@ -65,9 +65,10 @@ _XQueryColors(
 		def->blue = rgb->blue;
 		def->flags = DoRed | DoGreen | DoBlue;
 	    }
-	    Xfree((char *)color);
+	    Xfree(color);
 	}
-	else _XEatData(dpy, (unsigned long) nbytes);
+	else
+	    _XEatDataWords(dpy, rep.length);
     }
 }
 
@@ -81,9 +82,9 @@ XQueryColors(
     int n;
 
     if (dpy->bigreq_size > 0)
-	n = dpy->bigreq_size - (sizeof (xQueryColorsReq) >> 2) - 1;
+	n = (int) (dpy->bigreq_size - (sizeof (xQueryColorsReq) >> 2) - 1);
     else
-	n = dpy->max_request_size - (sizeof (xQueryColorsReq) >> 2);
+	n = (int) (dpy->max_request_size - (sizeof (xQueryColorsReq) >> 2));
 
     LockDisplay(dpy);
     while (ncolors >= n) {
